extends CharacterBody2D


const SPEED = 150.0
const JUMP_VELOCITY = -400.0
@export var DODGE_SPEED: float = 400
@export var dodge_time: float = 0.5
var current_dodge_time: float = 0
@export var dodge_duplicate_time: float = 0.05
var current_dodge_duplicate_time: float = 0
@export var duplicate_life_time: float = 0.3
var is_dodging = false
# Get the gravity from the project settings to be synced with RigidBody nodes.
var gravity = ProjectSettings.get_setting("physics/2d/default_gravity")


func _physics_process(delta):
	# Add the gravity.
	if not is_on_floor():
		velocity.y += gravity * delta

	# Handle Jump.
	if Input.is_action_just_pressed("ui_accept") and is_on_floor():
		velocity.y = JUMP_VELOCITY
	# Get the input direction and handle the movement/deceleration.
	# As good practice, you should replace UI actions with custom gameplay actions.
	
	var direction = Input.get_axis("ui_left", "ui_right")
	if !is_dodging && $TimerTakeDamage.is_stopped():
		# control the character
		if direction:
			$AnimationPlayer.play("walk")
			$AnimationPlayer.speed_scale = 2.0
			if direction > 0:
				$Sprite2D.scale.x = 1
			else:
				$Sprite2D.scale.x = -1
			velocity.x = direction * SPEED
		else:
			$AnimationPlayer.play("idle")
			$AnimationPlayer.speed_scale = 1.0
			velocity.x = move_toward(velocity.x, 0, SPEED)
	else:
		if is_dodging:
			velocity.x = lerp(velocity.x, 0.0, 0.1)
		else:
			$AnimationPlayer.play("idle")
			$AnimationPlayer.speed_scale = 1.0
			velocity.x = move_toward(velocity.x, 0, SPEED)
			
	if is_dodging:
		current_dodge_time += delta
		current_dodge_duplicate_time += delta
		if current_dodge_time >= dodge_time:
			current_dodge_duplicate_time = 0.0
			current_dodge_time = 0.0
			is_dodging = false
		if current_dodge_duplicate_time >= dodge_duplicate_time:
			current_dodge_duplicate_time = 0.0
			createDuplicate()
	
	if is_dodging == false && Input.is_action_just_pressed("dodge"):
		is_dodging = true
		if $Sprite2D.scale.x == 1:
			velocity.x = -DODGE_SPEED
		else:
			velocity.x = DODGE_SPEED
		createDuplicate()
		
	
	move_and_slide()
	pass
	
func createDuplicate():
	var duplicate = $Sprite2D.duplicate(true)
	duplicate.material = $Sprite2D.material.duplicate(true)
	duplicate.material.set_shader_parameter("opacity", 0.7)
	duplicate.material.set_shader_parameter("r", 0.0)
	duplicate.material.set_shader_parameter("g", 0.0)
	duplicate.material.set_shader_parameter("b", 0.8)
	duplicate.material.set_shader_parameter("mix_color", 0.6)
	duplicate.position.y += position.y
	if $Sprite2D.scale.x == -1:
		duplicate.position.x = position.x - duplicate.position.x
	else:
		duplicate.position.x += position.x
	duplicate.z_index -= 1
	get_parent().add_child(duplicate)
	await get_tree().create_timer(duplicate_life_time).timeout
	duplicate.queue_free()
	
	pass

func take_damage(iPos: Vector2, iDamageAmount: int):
	if !is_dodging && $TimerTakeDamage.is_stopped():
		$Sprite2D.material.set_shader_parameter("opacity", 0.7);
		$Sprite2D.material.set_shader_parameter("r", 1.0);
		$Sprite2D.material.set_shader_parameter("g", 0);
		$Sprite2D.material.set_shader_parameter("b", 0);
		$Sprite2D.material.set_shader_parameter("mix_color", 0.7);
		$TimerTakeDamage.start(0)
		if (iPos.x < position.x):
			velocity.x = 1100
		else:
			velocity.x = -1100
	pass


func _on_timer_take_damage_timeout():
	$TimerTakeDamage.stop()
	$Sprite2D.material.set_shader_parameter("opacity", 1.0);
	$Sprite2D.material.set_shader_parameter("mix_color", 0);
	pass # Replace with function body.
